package com.ruoyi.bst.articleCategory.service.impl;

import java.util.*;

import com.github.pagehelper.PageHelper;
import com.ruoyi.bst.area.domain.AreaQuery;
import com.ruoyi.bst.area.domain.AreaVO;
import com.ruoyi.bst.device.utils.DeviceUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.bst.articleCategory.mapper.ArticleCategoryMapper;
import com.ruoyi.bst.articleCategory.domain.ArticleCategory;
import com.ruoyi.bst.articleCategory.domain.ArticleCategoryVO;
import com.ruoyi.bst.articleCategory.domain.ArticleCategoryQuery;
import com.ruoyi.bst.articleCategory.service.ArticleCategoryService;
import org.springframework.transaction.annotation.Transactional;

/**
 * 文章分类Service业务层处理
 *
 * @author ruoyi
 * @date 2025-03-27
 */
@Service
public class ArticleCategoryServiceImpl implements ArticleCategoryService
{
    @Autowired
    private ArticleCategoryMapper articleCategoryMapper;

    /**
     * 查询文章分类
     *
     * @param id 文章分类主键
     * @return 文章分类
     */
    @Override
    public ArticleCategoryVO selectArticleCategoryById(Long id)
    {
        if (id == null) {
            return null;
        }
        ArticleCategoryQuery query = new ArticleCategoryQuery();
        query.setId(id);
        return this.selectOne(query);
    }

    public ArticleCategoryVO selectOne(ArticleCategoryQuery query){
            PageHelper.startPage(1, 1);
            List<ArticleCategoryVO> list = articleCategoryMapper.selectArticleCategoryList(query);
            return CollectionUtils.firstElement(list);
    }
    /**
     * 查询文章分类列表
     *
     * @param articleCategoryQuery 文章分类
     * @return 文章分类
     */
    @Override
    public List<ArticleCategoryVO> selectArticleCategoryList(ArticleCategoryQuery articleCategoryQuery)
    {
        return articleCategoryMapper.selectArticleCategoryList(articleCategoryQuery);
    }

    /**
     * 新增文章分类
     *
     * @param articleCategory 文章分类
     * @return 结果
     */
    @Override
    public int insertArticleCategory(ArticleCategory articleCategory)
    {
        //目标分类不允许子分类重名
        Boolean flag = articleCategoryMapper.checkByNameAndParetId(articleCategory);
        if (flag){
            ServiceUtil.assertion(true,"不允许插入重复分类");
        }
        //只允许两次分类
        if(articleCategory.getParentId()!= null) {
            Boolean hasParent = hasParent(articleCategory.getParentId());
            ServiceUtil.assertion(hasParent,"该分类下不允许再次分类");
        }
        articleCategory.setCreateTime(DateUtils.getNowDate());
        return articleCategoryMapper.insertArticleCategory(articleCategory);
    }

    /**
     * 修改文章分类
     *
     * @param articleCategory 文章分类
     * @return 结果
     */
    @Override
    public int updateArticleCategory(ArticleCategory articleCategory)
    {
        //目标分类不允许子分类重名
        Boolean flag = articleCategoryMapper.checkByNameAndParetId(articleCategory);
        if (flag){
            ServiceUtil.assertion(true,"目标分类下已经存在相同名称的分类");
        }
        //只允许两次分类
        if(articleCategory.getParentId()!= null) {
            Boolean hasParent = hasParent(articleCategory.getParentId());
            ServiceUtil.assertion(hasParent,"该分类下不允许再次分类");
        }
        return articleCategoryMapper.updateArticleCategory(articleCategory);
    }

    private Boolean hasParent(Long id){
        Long parentId = selectArticleCategoryById(id).getParentId();
        if (parentId != null){
            return true;
        }
        return false;
    }
    /**
     * 批量删除文章分类
     *
     * @param ids 需要删除的文章分类主键
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteArticleCategoryByIds(Long[] ids)
    {
        List<Long> cids = articleCategoryMapper.selectChildren(ids);
        List<Long> allIdsToDelete = new ArrayList<>();
        allIdsToDelete.addAll(cids);
        allIdsToDelete.addAll(Arrays.asList(ids));

        if (!allIdsToDelete.isEmpty()){
            return articleCategoryMapper.deleteArticleCategoryByIds(ids);
        }
        return 0;
    }

    /**
     * 删除文章分类信息
     *
     * @param id 文章分类主键
     * @return 结果
     */
    @Override
    public int deleteArticleCategoryById(Long id)
    {
        return articleCategoryMapper.deleteArticleCategoryById(id);
    }
}
