package com.ruoyi.bst.order.service.impl;

import java.time.LocalDateTime;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ruoyi.bst.app.domain.AppVO;
import com.ruoyi.bst.app.service.AppService;
import com.ruoyi.bst.area.domain.AreaVO;
import com.ruoyi.bst.area.service.AreaService;
import com.ruoyi.bst.areaJoin.service.AreaJoinService;
import com.ruoyi.bst.areaSub.domain.AreaSubVO;
import com.ruoyi.bst.areaSub.service.AreaSubService;
import com.ruoyi.bst.channel.domain.ChannelVO;
import com.ruoyi.bst.channel.service.ChannelService;
import com.ruoyi.bst.device.domain.DeviceVO;
import com.ruoyi.bst.device.service.DeviceIotService;
import com.ruoyi.bst.device.service.DeviceService;
import com.ruoyi.bst.locationLog.domain.vo.LocationLogPositionVO;
import com.ruoyi.bst.locationLog.service.LocationLogService;
import com.ruoyi.bst.order.domain.Order;
import com.ruoyi.bst.order.domain.OrderVO;
import com.ruoyi.bst.order.domain.bo.OrderCreateBO;
import com.ruoyi.bst.order.domain.bo.OrderEndBO;
import com.ruoyi.bst.order.domain.bo.OrderPayBO;
import com.ruoyi.bst.order.domain.dto.OrderCalcPrePriceDTO;
import com.ruoyi.bst.order.domain.dto.OrderCreateDTO;
import com.ruoyi.bst.order.domain.dto.OrderEndDTO;
import com.ruoyi.bst.order.domain.dto.OrderPayDTO;
import com.ruoyi.bst.order.domain.enums.OrderStatus;
import com.ruoyi.bst.order.domain.vo.OrderInParkingVO;
import com.ruoyi.bst.order.domain.vo.OrderPrePriceVO;
import com.ruoyi.bst.order.service.OrderConverter;
import com.ruoyi.bst.order.service.OrderService;
import com.ruoyi.bst.order.utils.OrderUtil;
import com.ruoyi.bst.suit.domain.SuitVO;
import com.ruoyi.bst.suit.domain.enums.SuitType;
import com.ruoyi.bst.suit.service.SuitService;
import com.ruoyi.bst.userApp.domain.UserAppVO;
import com.ruoyi.bst.userApp.service.UserAppService;
import com.ruoyi.common.core.domain.vo.UserVO;
import com.ruoyi.system.user.service.UserService;

@Service
public class OrderConverterImpl implements OrderConverter{

    @Autowired
    private SuitService suitService;

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private UserService userService;

    @Autowired
    private AreaJoinService areaJoinService;

    @Autowired
    private ChannelService channelService;

    @Autowired
    private AppService appService;

    @Autowired
    private UserAppService userAppService;

    @Autowired
    private DeviceIotService deviceIotService;

    @Autowired
    private AreaService areaService;

    @Autowired
    private AreaSubService areaSubService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private LocationLogService locationLogService;

    @Override
    public OrderPrePriceVO toOrderPrePriceVO(OrderCalcPrePriceDTO dto) {
        if (dto == null) {
            return null;
        }
        // 查询套餐
        SuitVO suit = suitService.selectSuitById(dto.getSuitId());
        // 计算价格
        return OrderUtil.calcPrePrice(suit);
    }

    @Override
    public OrderCreateBO toCreateBO(OrderCreateDTO dto) {
        if (dto == null) {
            return null;
        }
        OrderCreateBO bo = new OrderCreateBO();
        bo.setDto(dto);

        // 套餐
        SuitVO suit = suitService.selectSuitById(dto.getSuitId());
        bo.setSuit(suit);

        // 设备
        DeviceVO device = deviceService.selectDeviceById(dto.getDeviceId());
        bo.setDevice(device);

        // 用户
        bo.setUser(userService.selectUserById(dto.getUserId()));

        // 价格信息
        OrderPrePriceVO prePrice = OrderUtil.calcPrePrice(suit);
        bo.setPrePrice(prePrice);

        // 生成订单
        Order order = this.generateOrder(bo);
        bo.setOrder(order);

        return bo;
    }

    // 生成订单
    private Order generateOrder(OrderCreateBO bo) {
        if (bo == null) {
            return null;
        }
        Order order = new Order();
        // 基础信息
        order.setPayExpireTime(LocalDateTime.now().plusMinutes(10));

        // 用户信息
        UserVO user = bo.getUser();
        if (user != null) {
            order.setUserId(user.getUserId());
        }

        // 设备信息
        DeviceVO device = bo.getDevice();
        if (device != null) {
            order.setDeviceId(device.getId());
            order.setAreaId(device.getAreaId());
            order.setDeviceMac(device.getMac());
            order.setDeviceSn(device.getSn());
            order.setDeviceVehicleNum(device.getVehicleNum());
        }

        // 套餐信息
        SuitVO suit = bo.getSuit();
        if (suit != null) {
            order.setSuitId(suit.getId());
            order.setSuitType(suit.getType());
            order.setSuitName(suit.getName());
            order.setSuitFreeRideTime(suit.getFreeRideTime());
            order.setSuitRentalUnit(suit.getRentalUnit());
            order.setSuitRidingRule(suit.getRidingRule());
            order.setSuitStartRule(suit.getStartRule());
            order.setSuitIntervalRule(suit.getIntervalRule());
            order.setSuitSeconds(suit.getSeconds());
        }

        // 价格信息
        OrderPrePriceVO prePrice = bo.getPrePrice();
        if (prePrice != null) {
            order.setDepositFee(prePrice.getDepositPrice());
            order.setPayAmount(prePrice.getPayAmount());
        }

        // 订单状态
        if (SuitType.SHARE.getCode().equals(order.getSuitType())) {
            order.setStatus(OrderStatus.WAIT_PAY.getCode());
        } else {
            throw new RuntimeException("暂未开放租赁套餐");
        }

        return order;
    }

    @Override
    public OrderPayBO toPayBO(OrderVO order, OrderPayDTO dto) {
        if (dto == null || order == null) {
            return null;
        }

        Long channelId = dto.getChannelId();
        Long appId = dto.getAppId();

        OrderPayBO bo = new OrderPayBO();
        bo.setDto(dto);
        bo.setOrder(order);

        // 查询渠道
        ChannelVO channel = channelService.selectChannelByChannelId(channelId);
        bo.setChannel(channel);

        // 查询应用
        AppVO app = appService.selectAppById(appId);
        bo.setApp(app);

        if (order != null) {
            // 查询设备
            DeviceVO device = deviceService.selectDeviceById(order.getDeviceId());
            bo.setDevice(device);

            // 查询用户
            UserVO user = userService.selectUserById(order.getUserId());
            bo.setUser(user);

            // 查询用户应用信息
            if (user != null && app != null) {
                UserAppVO userApp = userAppService.selectByUserApp(user.getUserId(), app.getId());
                bo.setUserApp(userApp);
            }
        }


        return bo;
    }

    @Override
    public Order toPOByEnd(OrderVO order) {
        if (order == null) {
            return null;
        }
        // 需要更新的字段
        Order data = new Order();
        data.setStatus(order.getStatus());
        data.setManageFee(order.getManageFee());
        data.setDispatchFee(order.getDispatchFee());
        data.setRidingFee(order.getRidingFee());
        data.setTotalFee(order.getTotalFee());
        data.setEndTime(order.getEndTime());
        data.setEndAreaSubId(order.getEndAreaSubId());
        data.setDuration(order.getDuration());
        data.setDistance(order.getDistance());
        data.setReturnLon(order.getReturnLon());
        data.setReturnLat(order.getReturnLat());
        data.setReturnMode(order.getReturnMode());
        data.setReturnType(order.getReturnType());
        data.setEndAreaSubName(order.getEndAreaSubName());
        data.setEndReason(order.getEndReason());
        return data;
    }

    @Override
    public OrderEndBO toEndBO(OrderEndDTO dto) {
        if (dto == null) {
            return null;
        }

        OrderEndBO bo = new OrderEndBO();
        bo.setDto(dto);

        // 获取订单
        OrderVO order = orderService.selectOrderById(dto.getOrderId());
        bo.setOrder(order);
        if (order != null) {

            // 获取设备信息
            DeviceVO device = deviceService.selectDeviceById(order.getDeviceId());
            deviceIotService.refresh(device, null);
            bo.setDevice(device);

            if (device != null) {

                // 查询运营区
                AreaVO area = areaService.selectAreaById(device.getAreaId());
                bo.setArea(area);

                if (area != null) {
                    // 查询运营区的可还车的子区域列表
                    List<AreaSubVO> areaSubList = areaSubService.selectParkingAreaByAreaId(area.getId());
                    bo.setParkingAreaSubList(areaSubList);

                    // 停车点信息
                    OrderInParkingVO inParkingVO = OrderUtil.getInParkingVO(area, areaSubList, device, dto.getLon(), dto.getLat());
                    bo.setInParkingVO(inParkingVO);
                }
            }

            // 查询订单车辆轨迹列表
            List<LocationLogPositionVO> positionList = locationLogService.selectPositionListByOrderId(order.getId());
            bo.setPositionList(positionList);
        }   

        return bo;
    }

}
