package com.ruoyi.web.app;

import com.ruoyi.bst.device.domain.DeviceVO;
import com.ruoyi.bst.device.service.DeviceService;
import com.ruoyi.bst.fault.domain.Fault;
import com.ruoyi.bst.fault.domain.FaultQuery;
import com.ruoyi.bst.fault.domain.FaultVO;
import com.ruoyi.bst.fault.domain.enums.FaultHandleStatus;
import com.ruoyi.bst.fault.domain.enums.FaultStatus;
import com.ruoyi.bst.fault.service.FaultService;
import com.ruoyi.bst.order.service.OrderService;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * 故障申报前台
 */
@RestController
@RequestMapping("/app/fault")
public class AppFaultController extends BaseController {

    @Autowired
    private FaultService  faultService;

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private OrderService orderService;

    /**
     * 新增
     * @param fault
     * @return
     */
    @PostMapping
    public AjaxResult add(@RequestBody Fault fault) {
        faultService.insertFault(fault);
        return success();
    }


    @PutMapping
    public AjaxResult update(@RequestBody Fault fault) {
        FaultVO faultVO = faultService.selectFaultById(fault.getId());
        if (faultVO.getAppealStatus().equals(FaultStatus.PENGDING.getCode()) && fault.getHandle().equals(FaultHandleStatus.REJECTED.getCode())) {
            faultVO.setAppealStatus(FaultStatus.REJECTED.getCode());
        }else
        if (faultVO.getAppealStatus().equals(FaultStatus.PENGDING.getCode()) && fault.getHandle().equals(FaultHandleStatus.DO.getCode())) {
            faultVO.setAppealStatus(FaultStatus.REPAIRING.getCode());
        }else
        if (faultVO.getAppealStatus().equals(FaultStatus.REPAIRING.getCode()) && fault.getHandle().equals(FaultHandleStatus.DO.getCode())) {
            faultVO.setAppealStatus(FaultStatus.COMPLETED.getCode());
        }
        faultService.updateFault(faultVO);
        return success();
    }

    /**
     * 历史申报查询
     * @param query
     * @return
     */
    @GetMapping
    public TableDataInfo get(FaultQuery query) {
        startPage();
        query.setUserId(getUserId());
        List<FaultVO> list = faultService.selectFaultList(query);
        List<FaultVO> validList = new ArrayList<>();

        list.forEach(faultVO -> {
            DeviceVO deviceVO = deviceService.selectDeviceBySn(faultVO.getVehicleCode());
            // 查询的设备不能为空
            if (deviceVO == null){
                return;
            }
            Long orderId = deviceVO.getOrderId();
            LocalDateTime orderEndTime = orderService.selectOrderById(orderId).getEndTime();
            faultVO.setOrderEndTime(orderEndTime);
            validList.add(faultVO);
        });
        return getDataTable(validList);
        }
    }
